#!/bin/bash 

# This script constructs an approximated unfolding 
# for the dining philosophers example (finite version) using aunfold. 
# Input: dphil_finite.xml (in GTXL standard for graph transformation systems).
# Output: out.xml (in GXL standard for the Petri graph obtained by the construction of the approximated unfolding).

# It also generates a visualisation of the example: 
# pg2neato extracts the Petri net and hypergraph components of the Petri graph. 
# Input here: out.xml, output: neatoInput.petriNet / neatoInput.hypergraph files.
# By the means of the dot/neato programs, the following files are constructed: 
# out.pn.ps is a visualisation of the Petri net, and out.hg.ps is a visualisation of the hypergraph. 


if [ -f ../../example/dphil_finite.xml ]
then 

if [ -f out.xml ]
then 
rm out.xml
fi 

# approximated unfolding constructed
./../../bin/aunfold ../../example/dphil_finite.xml out.xml

echo "

An analysis of the Petri net underlying the approximated unfolding reveals
that no marking which consists of tokens exclusively in WL
or of tokens exclusively in WR is reachable from the initial marking.
Hence the system will never reach a deadlock.
This property cannot be proven using the current implementation.
"

if [ -f out.xml ]
then 


# ------ Visualisation ------
# extract the Petri net and Petri graph components and transform them to neato/dot input format
./../../bin/pg2neato -pn out.xml > neatoInput.petriNet
./../../bin/pg2neato -hg out.xml > neatoInput.hypergraph

# See Graphviz documentation at http://www.research.att.com/sw/tools/graphviz for additional details on dot and neato.
if [ -f neatoInput.petriNet ]
then
dot -Tps -Goverlap=false neatoInput.petriNet > out.pn.ps                
else echo "Missing neatoInput.petriNet file"
fi 

if [ -f neatoInput.hypergraph ]
then
neato -Tps -Goverlap=false neatoInput.hypergraph > out.hg.ps
else echo "Missing neatoInput.hypergraph file"
fi 
# ---------------------------


else  "Missing out.xml file!"
fi # out

else echo "Missing dphil_finite.xml file!"
fi # dphil_finite 
