#!/bin/bash 

# Using aunfold, this script constructs an approximated unfolding for the "resources" example. 
# Input: resources.xml (in GTXL standard for graph transformation systems).
# Output: out.xml (in GXL standard for the Petri graph obtained by the construction of the approximated unfolding).

# By the means of pg2lola convert out.xml to LoLa-format required by the backward reachability algorithm (bwra)

# For a given graph and a regular expression, the tool sponge finds conditions on the markings of the hypregraph
# which ensure that a path in the hypergraph matches the regular expression.
# Input: out.xml (Petri graph), regular.txt (regular expression) 
# Output: marking file (list of markings, which can be used in order to show that no graphs containing undesireable paths can be reached)

# We use the bwra tool in order to check if a marking is coverable.
# Input: out.xml (Petri graph) and marking file. Output on terminal whether the marking is coverable.

# this script also performs the visualisation of the example: 
# pg2neato extracts the Petri net and hypergraph components of the Petri graph. 
# Input: out.xml
# Output: neatoInput.petriNet / neatoInput.hypergraph files
# By the means of the dot/neato programs, the following files are constructed: 
# out.pn.ps as a visualisation of the Petri net 
# out.hg.ps as a visualisation of the hypergraph. 
if [ -f out.xml ]
then 
rm out.xml
fi 

if [ -f ../../example/resources.xml ]
then 

#approximated unfolding constructed
./../../bin/aunfold ../../example/resources.xml out.xml   

if [ -f out.xml ]
then 




if [ -f out_lola ]
then 
rm out_lola
fi

#convert to LoLa-format - will be needed for backward reachability algorithm (bwra)
./../../bin/pg2lola out.xml > out_lola

if [ -f out_lola ]
then

if [ -f marking ]
then 
rm marking
fi

if [ -f regular ]
then

#regular expression - stored in file regular
#given a graph and regular expression, sponge finds conditions on the markings of the hypergraph which ensure that a path in the hypergraph matches the regular expression
./../../bin/sponge out.xml < regular > marking              

if [ -f marking ]
then

#check whether a marking is coverable. 

./../../bin/bwra out_lola marking

else echo "Error in sponge: no marking file constructed!"
fi # marking

else echo "Missing regular file"
fi #regular
 
else echo "Error in pg2lola: no out_lola file constructed!"
fi # out_lola

# ------ Visualisation ------
#extract the Petri net and Petri graph and transform it to neato/dot input format
./../../bin/pg2neato -pn out.xml > neatoInput.petriNet
./../../bin/pg2neato -hg out.xml > neatoInput.hypergraph

#See Graphviz documentation at http://www.research.att.com/sw/tools/graphviz for additional details on dot and neato.
if [ -f neatoInput.petriNet ]
then
dot -Tps -Goverlap=false neatoInput.petriNet > out.pn.ps                
else echo "Missing neatoInput.petriNet file"
fi 

if [ -f neatoInput.hypergraph ]
then
neato -Tps -Goverlap=false neatoInput.hypergraph > out.hg.ps
else echo "Missing neatoInput.hypergraph file"
fi 
# ---------------------------



else  "Error in aunfold: no out.xml constructed!"
fi # out

else echo "Missing resources.xml file!"
fi # resources.xml
