
#include "ConverterToNeato.h"

#include <string>

namespace aunfold
{

ConverterToNeato::ConverterToNeato(GraphGrammar & ggr, string output)
{
   this->ggr = ggr;
   this->output= output;
}

ConverterToNeato::~ConverterToNeato()
{
	
}

/** convert initial graph and rules to dot format */
void ConverterToNeato::convert()
{
	// initial graph
	Hypergraph initial_graph = ggr.getStartGraph();

	string inis = output + (string)"/ini.dot";
	ofstream ini(inis.c_str());

	map<int, int> iniMap;
	
  writeDotFile(initial_graph, ini, iniMap);

	ini.close();

	// rules
	map<IDTYPE,TransformationRule> rules = ggr.getRules();

	int counter = 1;
	string sf;

	string sel = "l.dot";
	string ser = "r.dot";
	string sfl = output + "/";
		
	for(map<IDTYPE,TransformationRule>::iterator itR = rules.begin(); itR!=rules.end(); itR++)
	{
    // files
    ofstream ofl;
		ofstream ofr;
		
		char buf[256];
		sprintf(buf, "%d", counter);
    string sb(buf);

    string sl = sfl + sb + sel;
    string sr = sfl + sb + ser;

    ofl.open(sl.c_str());
    ofr.open(sr.c_str());

    // mapping
		map<int, int> mapleft;
		map<int, int> mapright;

		Hypergraph hgl = itR->second.getLeft();
		Hypergraph hgr = itR->second.getRight();

		set<IDTYPE> verticesleft = hgl.getVertices();
		set<IDTYPE> verticesright = hgr.getVertices();

		int count = 1;

    // calculate mapping
		for(set<IDTYPE>::iterator itV = verticesleft.begin(); itV!=verticesleft.end(); itV++)
    {
      if(verticesright.count(*itV)!=0)
      {
				mapleft[*itV] = count;
				mapright[*itV] = count;
		  }
		  else
		  {
				mapleft[*itV] = count;
				mapright[-(*itV)] = count;
		  }

		  count++;
    }

    // write
    writeDotFile(hgl, ofl, mapleft);
    writeDotFile(hgr, ofr, mapright);
    
    counter++;

    ofl.close();
    ofr.close();
		
	}
                                     
}

/** write a dot file for this hypergraph */
void ConverterToNeato::writeDotFile(Hypergraph & hg, ofstream & of, map<int, int> & nodeLabels)
{
  // vertices
	set<IDTYPE> vertices = hg.getVertices();

	of << "graph \"out\" {" << endl;

	for(set<IDTYPE>::iterator it = vertices.begin(); it!=vertices.end(); it++)
	{
		of << "_" << *it << " [shape=circle, width=.15, label=\"";

		if(nodeLabels[*it]!=0)
    {
			of << nodeLabels[*it];
    }
		
		of << "\"] \n";
	}

	// isolated nodes
	for(map<int, int>::iterator it = nodeLabels.begin(); it!=nodeLabels.end(); it++)
	{
		if(it->first < 0)
		{
			of << "_" << createNewID() << " [shape=circle, width=.15, label=\"";
			of << it->second;
			of << "\"] \n";
		}
			
	}

	// edges
	set<IDTYPE> edges = hg.getEdges();

  for(set<IDTYPE>::iterator it = edges.begin(); it!=edges.end(); it++)
  {
    of << "_" << *it << " [label=\"" << hg.getLabel(*it) << "\", shape=box] \n";

		list<IDTYPE> connectedV =  hg.getConnectedVertices(*it);

		int counter = 0;
    map<int, int> edgeLabelPos;
    const float PI = 3.1415;
    float angle = 0;
    int distance = 0;

    for(list<IDTYPE>::iterator itV = connectedV.begin(); itV!=connectedV.end(); itV++)
    {
      int pos = edgeLabelPos[*itV]+1;


      if ((pos % 2)  == 0)
      {
        angle = atan(tan(35.0*PI/180)/(pos/2))/PI*180;
        distance = pos / 2;
      }
      else
      {
        angle = - atan(tan(35.0*PI/180)/((pos+1)/2))/PI*180;
        distance = (pos+1)/2;
      }

      of << "_" << *it << " -- _" << *itV << "[taillabel=\"" << counter << "\", labelangle=";
			of << angle << ", labeldistance=" << distance << "]\n";

			counter++;
			edgeLabelPos[*itV]++;
		}
	}

  of << "}\n";

}

}
