#include "Hypergraph.h"
#include "Globals.h"

#include <assert.h>

using namespace std;

namespace aunfold
{




  //IDTYPE Hypergraph::lastUsedID(0);

  Hypergraph::Hypergraph()
  {
    
  }

  Hypergraph::Hypergraph(const Hypergraph& h)
    :    vertices(h.vertices),
	 edges(h.edges),
	 connectedV(h.connectedV),
	 connectedE(h.connectedE),
	 labels(h.labels),
	 idLabels(h.idLabels)
  {
    /*    vertices = h.vertices;
    edges = set<IDTYPE>(h.edges);
    connectedV = map<IDTYPE,list<IDTYPE> >(h.connectedV);
    connectedE = map<IDTYPE,multiset<IDTYPE> >(h.connectedE);
    for(map<IDTYPE,LABELTYPE>::const_iterator i = h.labels.begin();
        i!=h.labels.end();
        i++)
    {
       labels[(*i).first] = LABELTYPE((*i).second);
    }

    */
  }

  Hypergraph::~Hypergraph()
  {

  }  

  /** Adds a new vertex to the hypergraph. The ID of the created
      vertex is returned. */
  IDTYPE Hypergraph::addVertex()
  {
    IDTYPE newv = createNewID();
    vertices.insert(newv);
    connectedE.insert(pair<IDTYPE,multiset<IDTYPE> >(newv,multiset<IDTYPE>()));
    return newv;
  }

  IDTYPE Hypergraph::addVertex(IDTYPE id)
  {
    if(0==vertices.count(id))
      {
	
	vertices.insert(id);
	connectedE.insert(pair<IDTYPE,multiset<IDTYPE> >(id,multiset<IDTYPE>()));
      }
    return id;
  }
  
  
  /** Adds a new hyperedge to the hypergraph.
      connection is a list of vertices in the hypergraph.
      label is a label for the new edge.
      The ID of the new edge is returned.
  */

  IDTYPE Hypergraph::addEdge(list<IDTYPE> connection, LABELTYPE label)
  {
    IDTYPE newe = createNewID();
    edges.insert(newe);
    connectedV.insert(pair<IDTYPE,list<IDTYPE> >(newe,connection));
    labels.insert(pair<IDTYPE,LABELTYPE>(newe,LABELTYPE(label)));

    for(list<IDTYPE>::iterator i = connection.begin();
	i != connection.end();
	i++)
      {
	assert(1==vertices.count(*i)); // connection must not contain other IDs than vertices of this hypergraph
	// find multiset of edges connected to vertex *i
	map<IDTYPE,multiset<IDTYPE> >::iterator j = connectedE.find(*i);
	(*j).second.insert(newe);
      }
    return newe;
  }

  IDTYPE Hypergraph::addEdge(list<IDTYPE> connection, LABELTYPE label, IDTYPE id)
  {
    if(0==edges.count(id))
      {
	edges.insert(id);
	connectedV.insert(pair<IDTYPE,list<IDTYPE> >(id,connection));
	labels.insert(pair<IDTYPE,LABELTYPE>(id,LABELTYPE(label)));
	
	for(list<IDTYPE>::iterator i = connection.begin();
	    i != connection.end();
	    i++)
	  {
	    assert(1==vertices.count(*i)); // connection must not contain other IDs than vertices of this hypergraph
	    // find multiset of edges connected to vertex *i
	    map<IDTYPE,multiset<IDTYPE> >::iterator j = connectedE.find(*i);
	    (*j).second.insert(id);
	  }
      }
    return id;
  }





  void Hypergraph::uniongraphs(const Hypergraph& h)
  {
    // add vertices of h to this:
    for(set<IDTYPE>::iterator i = h.vertices.begin();
        i != h.vertices.end();
	i++)
      {
	// assert vertice set are disjunct
	assert(0 == vertices.count(*i) );
	vertices.insert(*i);
      }
    // add edges and labels of h to this:
    for(set<IDTYPE>::const_iterator j = h.edges.begin();
	j != h.edges.end();
	j++)
      {
	// assert edge sets are disjunct
	assert (0 == edges.count(*j));
	edges.insert(*j);
	labels.insert(*(h.labels.find(*j)));
      }
    // add connectivity mapping
    for(map<IDTYPE,list<IDTYPE> >::const_iterator k = h.connectedV.begin();
	k != h.connectedV.end();
	k++)
      {
	connectedV.insert(*k);
      }
    for(map<IDTYPE,multiset<IDTYPE> >::const_iterator l = h.connectedE.begin();
	l != h.connectedE.end();
	l++)
      {
	connectedE.insert(*l);
      }
  }

  LABELTYPE Hypergraph::getLabel(IDTYPE  edge) const
  {
    assert(1 == edges.count(edge));

    return (*(labels.find(edge))).second;
  }

  std::set<IDTYPE> Hypergraph::getVertices() 
  {
    return vertices;
  }

   /** set id label */
   void Hypergraph::setIdLabel(IDTYPE id, std::string ls)
   {
		 idLabels[id]=ls;
	 }

	 /** get id label */
	 std::string Hypergraph::getIdLabel(IDTYPE id)
	 {
	 if(idLabels.find(id)!=idLabels.end())
			 return idLabels[id];
	 else return "";
	 }
  
  std::set<IDTYPE> Hypergraph::getEdges()
  {
    return edges;
  }
  
  list<IDTYPE> Hypergraph::getConnectedVertices(IDTYPE edge) const
  {
    assert(1 == edges.count(edge));
    
    return (*(connectedV.find(edge))).second;
  }

  multiset<IDTYPE> Hypergraph::getConnectedEdges(IDTYPE vertex) const
  {
    assert (1 == vertices.count(vertex));

    return (*(connectedE.find(vertex))).second;
  }

  bool Hypergraph::setDepth(IDTYPE id, long depth, VERTEDGE v)
  {
      if(v==EDGE)
      	depthmapEdges[id] = depth;
      else
				depthmapVertices[id] = depth;
				 
			return true;
  }
  bool Hypergraph::getDepth(IDTYPE id, long * pdepth, VERTEDGE v)
  {
		*pdepth = 0;		
    // edge
    if(v==EDGE)
    {
    std::set<IDTYPE>::iterator it = edges.find(id);

		if(it==edges.end())
			return false;
		else
		{
			*pdepth = depthmapEdges[id];
			return true;
		}
		}
		// vertice
		else
		{
    std::set<IDTYPE>::iterator it = vertices.find(id);

		if(it==vertices.end())
			return false;
		else
		{
			*pdepth = depthmapVertices[id];
			return true;
		}			
		}
  }

  void Hypergraph::initDepth()
  {
   for(std::set<IDTYPE>::iterator it = edges.begin(); it != edges.end(); it++)
			depthmapEdges[(*it)] = 0;
			
		for(std::set<IDTYPE>::iterator it = vertices.begin(); it != vertices.end(); it++)
			depthmapVertices[(*it)] = 0;
  }

  std::map<IDTYPE, long> Hypergraph::getDepthTable(VERTEDGE v)
  {
		if(v==EDGE)
			return depthmapEdges;
		else
			return depthmapVertices;
  }

  void Hypergraph::setDepthTable(std::map<IDTYPE, long> & depthtable, VERTEDGE v)
  {
    if(v==EDGE)
    {
			depthmapEdges.empty();
			depthmapEdges = depthtable;
		}
		else
		{
			depthmapVertices.empty();
			depthmapVertices = depthtable;
		}
  }
  
  map<IDTYPE,IDTYPE> Hypergraph::findMatching(Hypergraph& h, MatchHandler& handler, long depth, long unfdepth)
  {
    // initialise datastructures
    stack<set<IDTYPE>::iterator> mappings;

    stack<IDTYPE> unmapped_edges;

    for(set<IDTYPE>::iterator i = h.edges.begin();
	i != h.edges.end();
	i++)
     unmapped_edges.push(*i);

    map<IDTYPE,IDTYPE> phi_v;
    map<IDTYPE,IDTYPE> phi_e;

    
    map<IDTYPE,IDTYPE> result;
    if(findNextMatching(h,mappings,unmapped_edges,phi_v,phi_e,handler, depth, unfdepth)) // Does a matching exist?
      {
	// return union of phi_v and phi_e
	for(map<IDTYPE,IDTYPE>::iterator v = phi_v.begin();
	    v != phi_v.end();
	    v++)
	  result.insert(*v);
	for(map<IDTYPE,IDTYPE>::iterator e = phi_e.begin();
	    e != phi_e.end();
	    e++)
	  result.insert(*e);
	return result;
      }
      else // no matching found
      {
	// findNextMatching should have cleared phi and mappings
	assert(0 == phi_v.size());
	assert(0 == phi_e.size());
	assert(0 == mappings.size());
	assert(unmapped_edges.size() == h.edges.size());
	return result; // return an empty map
      }
  }


  // private
  bool Hypergraph::findNextMatching(Hypergraph &h,
				    stack<set<IDTYPE>::iterator> &mappings,
				    stack<IDTYPE> &unmapped_edges,
				    map<IDTYPE,IDTYPE> &phi_v,
				    map<IDTYPE,IDTYPE> &phi_e,
				    MatchHandler& matchhandler,
				    long depth,
				    long unfdepth)
  {
    // a matching was found?
    if(0 == unmapped_edges.size())
      {
	map<IDTYPE,IDTYPE> thematching;
	for(map<IDTYPE,IDTYPE>::iterator v = phi_v.begin();
	    v != phi_v.end();
	    v++)
	  thematching.insert(*v);
	for(map<IDTYPE,IDTYPE>::iterator e = phi_e.begin();
	    e != phi_e.end();
	    e++)
	  thematching.insert(*e);	
	return matchhandler.useThisMatch(thematching);
      }
    else // the matching is not yet complete
      {
	// edges and vertices in *this are named with prefix g,
	// edges and vertices in h are named with prefix h
	// get the next edge to be mapped
	IDTYPE hedge = unmapped_edges.top();
  	unmapped_edges.pop();
	// iterator on mappings stack iterates through all edges of g
	// to find an appropriate mapping
	mappings.push(edges.begin());
	// label of hedge:
	LABELTYPE hedgelabel = h.getLabel(hedge);
	// all vertices connected to the edge to be mapped
	list<IDTYPE> hcV = h.getConnectedVertices(hedge);
	// iterate mappings.top() throung all edges of g
	while(mappings.top() != edges.end())
	  {
	    IDTYPE gedge = *(mappings.top());
	    // label of gedge:
	    LABELTYPE gedgelabel = getLabel(gedge);
 	    // all vertices connected to the mapping target
	    list<IDTYPE> gcV = getConnectedVertices(gedge);
	    // the label and cardinality of matching edges must be the same
      if((hedgelabel != gedgelabel) || (gcV.size() != hcV.size()))
	      {
		(mappings.top())++; // try another edge
		continue; 
	      }
	    // store all new vertice mappings to delete them later
	    list<IDTYPE> newMappedVertices;
	    bool vertexMatchingFailed(false);
	    

	    list<IDTYPE>::iterator hv = hcV.begin();
	    list<IDTYPE>::iterator gv = gcV.begin();
	    
	    while((!vertexMatchingFailed) && hv != hcV.end())
	    {
	      
		map<IDTYPE,IDTYPE>::iterator phi_v_it = phi_v.find(*hv);
		if(phi_v.end() == phi_v_it) // vertex is not yet mapped
		  {
		    newMappedVertices.push_back(*hv);
		    phi_v.insert(pair<IDTYPE,IDTYPE>(*hv,*gv));
		  }
		else //vertex is already mapped
		  if((*phi_v_it).second != *gv) //mismatch, abort
		    vertexMatchingFailed = true;
		gv++;
		hv++;
	    }
	    // now all vertices are mapped, or vertexMatchingFailed==true
	    if(!vertexMatchingFailed)
	      {
		phi_e.insert(pair<IDTYPE,IDTYPE>(hedge,gedge));
		if(findNextMatching(h,mappings,unmapped_edges,phi_v,phi_e,matchhandler, depth, unfdepth)) return true;
		phi_e.erase(hedge);
	      }
	    // undo all changes in phi_v
	    for(list<IDTYPE>::iterator v = newMappedVertices.begin();
		v != newMappedVertices.end();
		v++)
	      phi_v.erase(*v);
	    (mappings.top())++; // try next edge mapping
	  
	  }
	unmapped_edges.push(hedge);
	mappings.pop();
	
	return false; // nothing found
      }
   
  }


  ostream& operator<<(ostream& os, Hypergraph& hg)
  {
    os << "vertices: ";
    for(set<IDTYPE>::iterator i = hg.vertices.begin();
	i != hg.vertices.end();
	i++)
      os << (*i) << " ";
    os << "  edges: ";
    for(set<IDTYPE>::iterator i = hg.edges.begin();
	i!= hg.edges.end();
	i++)
      {
	os << (*i) << " <" << hg.getLabel(*i) << ">(";
	map<IDTYPE,list<IDTYPE> >::iterator j = hg.connectedV.find(*i);
	for(list<IDTYPE>::iterator k = (*j).second.begin();
	    k != (*j).second.end();
	    k++)
	  os << (*k) << " ";
	os << ") ";
      }
    return os;
  }
	


}


