
#include "Relation.h"
#include "Petrigraph.h"
#include "PTNet.h"

#include <list>
#include <map>



using namespace std;

namespace aunfold
{
  Relation::Relation(Petrigraph* pg)
    : petrigraph(pg)
  {
  }

  void Relation::addVertexRel(IDTYPE vertex1,IDTYPE vertex2)
  {
    map<IDTYPE,set<IDTYPE> >::iterator v1rel = vrel.find(vertex1);
    if(vrel.end() == v1rel)
      {
	vrel.insert(pair<IDTYPE,set<IDTYPE> >(vertex1,set<IDTYPE>()));
	v1rel = vrel.find(vertex1);
      }
    (*v1rel).second.insert(vertex2);
  }

  void Relation::addEdgeRel(IDTYPE edge1, IDTYPE edge2)
  {
    map<IDTYPE,set<IDTYPE> >::iterator e1rel = erel.find(edge1);
    if(erel.end() == e1rel)
      {
	erel.insert(pair<IDTYPE,set<IDTYPE> >(edge1,set<IDTYPE>()));
	e1rel = erel.find(edge1);
      }
    (*e1rel).second.insert(edge2);
  }


  void Relation::addTransitionRel(IDTYPE transition1, IDTYPE transition2)
  {
    map<IDTYPE,set<IDTYPE> >::iterator t1rel = trel.find(transition1);
    if(trel.end() == t1rel)
      {
	trel.insert(pair<IDTYPE,set<IDTYPE> >(transition1,set<IDTYPE>()));
	t1rel = trel.find(transition1);
      }
    (*t1rel).second.insert(transition2);
  }

  bool Relation::isVertexInRel(IDTYPE vertex1, IDTYPE vertex2)
  {
    map<IDTYPE,set<IDTYPE> >::iterator i = vrel.find(vertex1);
    if(vrel.end() == i) return false;
    return 1== (*(i)).second.count(vertex2);
  }
  bool Relation::isEdgeInRel(IDTYPE edge1, IDTYPE edge2)
  {
    map<IDTYPE,set<IDTYPE> >::iterator i = erel.find(edge1);
    if(erel.end() == i) return false;
    return 1== (*(i)).second.count(edge2);

  }
  bool Relation::isTransitionInRel(IDTYPE transition1, IDTYPE transition2)
  {
    map<IDTYPE,set<IDTYPE> >::iterator i = trel.find(transition1);
    if(trel.end() == i) return false;
    return 1== (*(i)).second.count(transition2);
    
  }


  
  std::map<IDTYPE,IDTYPE>  Relation::getExtendingEquivRelation()
  {
    // make shure every vertex, edge, or transition is in relation with itself
    {
      set<IDTYPE> v = petrigraph->getHypergraph().getVertices();
      for(set<IDTYPE>::iterator i=v.begin(); i!=v.end(); i++)
	addVertexRel(*i,*i);
      set<IDTYPE> e = petrigraph->getHypergraph().getEdges();
      for(set<IDTYPE>::iterator i=e.begin(); i!=e.end(); i++)
	addEdgeRel(*i,*i);
      
      list<PTNetTransition> t = petrigraph->getPTNet().getTransitions();
      for(list<PTNetTransition>::iterator i = t.begin(); i!=t.end(); i++)
	addTransitionRel((*i).getID(),(*i).getID());
    }
    

    // extend to a closed equivalence relation
    // iterate until no extension could be performed
    bool isClosedEquivRelation = true;
    do{
      isClosedEquivRelation = true;
      // for each pair of transitions:
      list<PTNetTransition> transitions = petrigraph->getPTNet().getTransitions();
      for(list<PTNetTransition>::iterator t1 = transitions.begin();
	  t1 != transitions.end();
	  t1++)
	for(list<PTNetTransition>::iterator t2 = transitions.begin();
	    t2!=transitions.end();
	    t2++)
	  {
	    IDTYPE t1ID = (*t1).getID();
	    IDTYPE t2ID = (*t2).getID();
	    if(isTransitionInRel(t1ID,t2ID)) // is t1 in relation to t2?
	      {
		// Def. 7, (4): t1~t2 => for all e in (E_l union E_r): mu(t1)(e)~mu(t2)(e)
		Hypergraph lefthandside =petrigraph->getGraphGrammar().getRule((*t1).getAssociatedRule()).getLeft();
		set<IDTYPE> e_lefthandside = lefthandside.getEdges();
		for(set<IDTYPE>::iterator lhsit = e_lefthandside.begin();
		    lhsit != e_lefthandside.end();
		    lhsit++)
		  {
		    IDTYPE id1 = petrigraph->mu((*t1).getID(),*lhsit);
		    IDTYPE id2 = petrigraph->mu((*t2).getID(),*lhsit);
		    if(!isEdgeInRel(id1,id2))
		      //0==(*(erel.find(id1))).second.count(id2))
		      {
			isClosedEquivRelation=false;  // we need to change something
			addEdgeRel(id1,id2);
			//(*(erel.find(id1))).second.insert(id2);
		      }
		  }
		// same for right hand side:
		Hypergraph righthandside =petrigraph->getGraphGrammar().getRule((*t1).getAssociatedRule()).getRight();
		set<IDTYPE> e_righthandside = righthandside.getEdges();
		for(set<IDTYPE>::iterator rhsit = e_righthandside.begin();
		    rhsit != e_righthandside.end();
		    rhsit++)
		  {
		    IDTYPE id1 = petrigraph->mu((*t1).getID(),*rhsit);
		    IDTYPE id2 = petrigraph->mu((*t2).getID(),*rhsit);
		    if(!isEdgeInRel(id1,id2))
		      //0==(*(erel.find(id1))).second.count(id2))
		      {
			isClosedEquivRelation=false;  // we need to change something
			addEdgeRel(id1,id2);
			//(*(erel.find(id1))).second.insert(id2);
		      }
		  }

	      }
	    else
	      {
		// def. 7, (3): p_N(t1)==p_N(t2) && for all e in E_L: mu(t1)(e)~mu(t2)(e) => t1~t2
		
		if((*t1).getAssociatedRule() == (*t2).getAssociatedRule())
		  {
		    // E_L:
		    set<IDTYPE> edges = petrigraph->getGraphGrammar().getRule((*t1).getAssociatedRule()).getLeft().getEdges();
		    bool ok = true;
		    for(set<IDTYPE>::iterator i=edges.begin();
			ok && i!= edges.end();
			i++)
		      {
			if(petrigraph->mu((*t1).getID(),*i) != petrigraph->mu((*t2).getID(),*i))
			  ok = false;
		      }
		    if(ok) // then t1~t2
		      {			
			addTransitionRel((*t1).getID(),(*t2).getID());
			isClosedEquivRelation = false;
		      }
		  } // if p_N(t1)==p_N(t2)
	      }// else (t1!~t2)
	  } // for each relation

      // def. 7, (5) e1~e2 && c_g(e1) == v1..vm && c_g(e2)==w1..wm => v_i~w_i
      for(map<IDTYPE,set<IDTYPE> >::iterator erel_it = erel.begin();
	  erel_it != erel.end();
	  erel_it++)
	{
	  IDTYPE e1 = (*erel_it).first;
	  list<IDTYPE> v = petrigraph->getHypergraph().getConnectedVertices(e1);
	  
	  for(set<IDTYPE>::iterator e2_it = (*erel_it).second.begin();
	      e2_it != (*erel_it).second.end();
	      e2_it++)
	    {
	      IDTYPE e2 = *e2_it;
	      // now e1 ~ e2
	      list<IDTYPE> w = petrigraph->getHypergraph().getConnectedVertices(e2);
	      if(v.size() == w.size()) // same arity?
		{
		  list<IDTYPE>::iterator v_it = v.begin();
		  list<IDTYPE>::iterator w_it = w.begin();
		  while(v_it != v.end())
		    {
		      if(!isVertexInRel(*v_it,*w_it))
			//0==(*(vrel.find(*v_it))).second.count(*w_it)) // v_i!~w_i
			{
			  isClosedEquivRelation = false;
			  addVertexRel(*v_it,*w_it);
			  //(*(vrel.find(*v_it))).second.insert(*w_it);
			}  // if
		      v_it++;
		      w_it++;
		    } // while
		}
	    } // for each edge relation
	} // for each edge

      // now ensure the relation is an equivalence relation
 // vertices:
      for(map<IDTYPE,set<IDTYPE> >::iterator i = vrel.begin();
	  i != vrel.end();
	  i++)
	{
	  // v1 ~ v1
	  if(0==(*i).second.count((*i).first)){
	    isClosedEquivRelation = false;
	    (*i).second.insert((*i).first);
	  }

	  for(set<IDTYPE>::iterator j = (*i).second.begin();
	      j != (*i).second.end();
	      j++)
	    {
	      // v1 ~ v2 => v2 ~ v1
	      map<IDTYPE,set<IDTYPE> >::iterator j2 = vrel.find(*j);
	      if(vrel.end() == j2) 
		{
		  vrel[*j]=set<IDTYPE>();
		  j2=vrel.find(*j);
		}
	      if(!isVertexInRel(*j,(*i).first))
	      {
		isClosedEquivRelation = false;
		addVertexRel(*j,(*i).first);
		j2 = vrel.find(*j);
	      }
	      
	      // v1 ~ v2 && v2 ~ v3 => v1 ~ v3
	      for(set<IDTYPE>::iterator k = (*j2).second.begin();
		  k != (*j2).second.end();
		  k++)
		{
		  if(!isVertexInRel((*i).first,*k))
		    {
		      isClosedEquivRelation = false;
		      addVertexRel((*i).first,*k);
		    }
		}
	    }
	}

// edges:
      for(map<IDTYPE,set<IDTYPE> >::iterator i = erel.begin();
	  i != erel.end();
	  i++)
	{
	  // e1 ~ e1
	  if(0==(*i).second.count((*i).first)){
	    isClosedEquivRelation = false;
	    (*i).second.insert((*i).first);
	  }

	  for(set<IDTYPE>::iterator j = (*i).second.begin();
	      j != (*i).second.end();
	      j++)
	    {
	      // e1 ~ e2 => e2 ~ e1
	      map<IDTYPE,set<IDTYPE> >::iterator j2 = erel.find(*j);
	      if(erel.end() == j2) 
		{
		  erel[*j]=set<IDTYPE>();
		  j2=erel.find(*j);
		}

	      if(!isEdgeInRel(*j,(*i).first))
	      {
		isClosedEquivRelation = false;
		addEdgeRel(*j,(*i).first);
		j2 = erel.find(*j);
	      }
		 
	      // e1 ~ e2 && e2 ~ e3 => e1 ~ e3
	      for(set<IDTYPE>::iterator k = (*j2).second.begin();
		  k != (*j2).second.end();
		  k++)
		{
		  if(!isEdgeInRel((*i).first,*k))
		    {
		      isClosedEquivRelation = false;
		      addEdgeRel((*i).first,*k);
		    }
		}
	    }
	}

// transitions
      for(map<IDTYPE,set<IDTYPE> >::iterator i = trel.begin();
	  i != trel.end();
	  i++)
	{
	  // t1 ~ t1
	  if(0==(*i).second.count((*i).first)){
	    isClosedEquivRelation = false;
	    (*i).second.insert((*i).first);
	  }

	  for(set<IDTYPE>::iterator j = (*i).second.begin();
	      j != (*i).second.end();
	      j++)
	    {
	      // t1 ~ t2 => t2 ~ t1
	      map<IDTYPE,set<IDTYPE> >::iterator j2 = trel.find(*j);
	      if(trel.end() == j2) 
		{
		  trel[*j]=set<IDTYPE>();
		  j2=trel.find(*j);
		}

	      if(!isTransitionInRel(*j,(*i).first))
	      {
		isClosedEquivRelation = false;
		addTransitionRel(*j,(*i).first);
		j2 = trel.find(*j);
	      }
		 
	      // t1 ~ t2 && t2 ~ t3 => t1 ~ t3
	      for(set<IDTYPE>::iterator k = (*j2).second.begin();
		  k != (*j2).second.end();
		  k++)
		{
		  if(!isTransitionInRel((*i).first,*k))
		    {
		      isClosedEquivRelation = false;
		      addTransitionRel((*i).first,*k);
		    }
		}
	    }
	}


    } while (!isClosedEquivRelation);
    
    // create map from old IDs to equivalence class IDs:
    map<IDTYPE,IDTYPE> result;
    
    for(map<IDTYPE,set<IDTYPE> >::iterator i = vrel.begin();
	i != vrel.end();
	i++)
      {
	if(0==result.count((*i).first)) // no ID for this vertex yet
	  {
	    IDTYPE newID = createNewID();

	  // calculate also the min depth
    long min_depth = 2147483647; // MAXLONG
    long current_depth;

			for(set<IDTYPE>::iterator j = (*i).second.begin();
		j != (*i).second.end();
		j++)
		{
	      result.insert(pair<IDTYPE,IDTYPE>(*j,newID));

	      petrigraph->getHypergraph().getDepth(*j, &current_depth, VERTICE);
	      if(current_depth<min_depth)
						min_depth = current_depth;
	  }
    petrigraph->getHypergraph().setDepth(newID, min_depth, VERTICE);
	    	    
	    for(set<IDTYPE>::iterator j = (*i).second.begin();
		j != (*i).second.end();
		j++)
	      result.insert(pair<IDTYPE,IDTYPE>(*j,newID));
	  }
      }
      
    for(map<IDTYPE,set<IDTYPE> >::iterator i = erel.begin();
	i != erel.end();
	i++)
      {
				
	if(0==result.count((*i).first)) // no ID for this edge yet
	  {
	    IDTYPE newID = createNewID();

     // calculate also the min depth
     
     long min_depth = 2147483647; // MAXLONG
     long current_depth;    
	      
			for(set<IDTYPE>::iterator j = (*i).second.begin();
		j != (*i).second.end();
		j++)
		{
	      result.insert(pair<IDTYPE,IDTYPE>(*j,newID));

	      petrigraph->getHypergraph().getDepth(*j, &current_depth, EDGE);
	      if(current_depth<min_depth)
						min_depth = current_depth;
	  }
    petrigraph->getHypergraph().setDepth(newID, min_depth, EDGE);

	  }
    }
    
    for(map<IDTYPE,set<IDTYPE> >::iterator i = trel.begin();
	i != trel.end();
	i++)
      {
	if(0==result.count((*i).first)) // no ID for this transition yet
	  {
	    IDTYPE newID = createNewID();
	    
	    for(set<IDTYPE>::iterator j = (*i).second.begin();
		j != (*i).second.end();
		j++)
	      result.insert(pair<IDTYPE,IDTYPE>(*j,newID));
	  }
      }
    
    return result;
  }
  
  

  
  
}
