#ifndef RELATION_H
#define RELATION_H

#include "Globals.h"

#include <map>
#include <set>

namespace aunfold
{
  class Petrigraph; // forward declaration
  /** Representation of an equivalence relation on a Petri graph as defined in def. 7 */
  class Relation
    {
    public:
      Relation(Petrigraph* pg);
      /** add vertex1 ~ vertex2 to the relation */
      void addVertexRel(IDTYPE vertex1,IDTYPE vertex2);
      /** add edge1 ~ edge2 to the relation.
       *  The caller must take care that labels of edge1 and edge2
       *  are identical to keep the relation consistent. */
      void addEdgeRel(IDTYPE edge1, IDTYPE edge2);
      /** add transition1 ~ transition2 to the relation
       *  The caller must take care that transition rules attached
       *  to transition1 and transition2 are identical to keep the
       *  relation consistent. */
      void addTransitionRel(IDTYPE transisition1, IDTYPE transition2);

      bool isVertexInRel(IDTYPE vertex1, IDTYPE vertex2);
      bool isEdgeInRel(IDTYPE edge1, IDTYPE edge2);
      bool isTransitionInRel(IDTYPE transition1, IDTYPE transition2);

      /** extend the relation to the least possible closed equivalence relation,
       *  assign a new ID to each equivalence class and return a mapping
       *  from the old IDs to the equivalence class IDs. */
      std::map<IDTYPE,IDTYPE>  getExtendingEquivRelation();

    private:

      Petrigraph* petrigraph;
      
      std::map<IDTYPE,std::set<IDTYPE> > vrel;
      std::map<IDTYPE,std::set<IDTYPE> > erel;
      std::map<IDTYPE,std::set<IDTYPE> > trel;
      
      
    }
  ;
}




#endif
