/* Content:
   - aunfold.TransformationRule
*/

#ifndef TRANSFORMATIONRULE
#define TRANSFORMATIONRULE

#include <string>

#include "Hypergraph.h"
#include "Globals.h"

namespace aunfold
{
  /** Transformation Rule of a Hypergraph Transformation System
   *  A subgraph matching of the left-hand side is replaced by
   * the right-hand side. The vertex
   * mapping from lhs to rhs is the identity of vertex IDs.
   * Lhs vertices without a corresponding rhs vertex are not
   * removed.
   * Every rule is identified by a unique ID.
   */


  class  TransformationRule
  {
  public:
    TransformationRule(Hypergraph l, Hypergraph r, std::string name);
    TransformationRule(const TransformationRule& r);
    ~TransformationRule();
    Hypergraph& getLeft();
    Hypergraph& getRight();
    IDTYPE getID();
    std::string getName();

    friend std::ostream& operator<<(std::ostream& os, TransformationRule& tr);
    
  private:
    Hypergraph left;
    Hypergraph right;
    IDTYPE ID;
    std::string rulename;
  };

}



#endif
