#include <map>
#include <iostream>
#include <vector>
#include <sstream>
#include "stringMap.h"

using namespace std;

#ifndef CONSTRAINT_H
#define CONSTRAINT_H

/* A constraint is similar to a marking, 
   a mapping from Places P to Integers,
   but a constraint represents a set of markings:
   constraint_m represents all markings m' which
   are greater equal m.
*/
class Constraint {
 protected:
  map<int,int> ct; // internal container
  map<int,int>::iterator i; // internal iterator
  vector<string> ltr; // list of fired transitions
 public:
  Constraint(); // constructor

  bool empty();  // is this constraint empty?

  /* update and access methods: */
  
  // set the marks at place specified by key to given value
  void set(int key,int value); 

  // get the number of marks at given place
  int get(int key);

  // delete all marks at given place
  void del(int key);
  
  // return string-representation of this constraint
  string asString(stringMap* placeNames);
  
  // return a new copy of this constraint
  Constraint* copy();

  /* comparision methods */
  
  // is this constraint smaller as given constraint at place given by index?
  bool smallerAs(Constraint* in, int index);

  // is this constraint smaller eqaual as given constraint?
  bool smallerEqualAs(Constraint* in);
  
  // shortcuts for method from above
  bool operator<=(Constraint* in) {return this->smallerEqualAs(in); }
  
  void addTransition(string trans);
  
  int getTransCount();
  
  string getTransition(int number);
};

#endif
