#include "Constraint.h"
#include "ConstraintSet.h"
#include "PredicateTransformer.h"
#include "PredicateTransformerSet.h"
#include "stringMap.h"

#ifndef InputCvs_H
#define InputCvs_H

/* class InputCvs: read Petri-Net description from files
   and provide Final-Set of Constraints,
   Initial-Set of Constraints and PredicateTransformerSet.
*/
class InputCvs {
  // variables, to store parsed information:
  // places, markings, transitions, place and transition names.
  ConstraintSet* final;
  ConstraintSet* initial;
  PredicateTransformerSet* ptSet;
  stringMap* placeNames;
  stringMap* transformerNames;

  // incr. info messages
  bool verbose;
  
  // methods, which actually do the parsing
  bool addPredicateTransformer(string in);
  void addConstraint(ConstraintSet* ctSet,string in);
  void setPlaces(string in);
  
  // helper methods
  void error_exit(const string in);
  string readFile(const char* file);
  
 public:
  InputCvs();
  void setVerbose() { verbose = true; }  
  
  // initiate parsing on files
  bool run(const char* file1,const char* file2);
  
  // provide parsed markings and transitions
  PredicateTransformerSet* getPredicateTransformerSet();
  ConstraintSet* getFinalConstraintSet();
  ConstraintSet* getInitialConstraintSet();
  stringMap* getTransformerNames();
  stringMap* getPlaceNames();
  
  // provide parsed markings and transitions as string
  string asString();
};
#endif
