/* Main program using the Backward Reachability Algorithm.
   2 Files in LoLA-Format are needed, 
   the first having the transition-net and initial marking,
   the second file having the final- or test-marking.
*/


#include "InputCvs.h"
#include "Cvs.h"

int main(int argc, char* argv[]) {

  bool verb = false; // verbose messages  
  bool quiet = false; // quiet mode
  char* file1;
  char* file2;
  string verbose = "-v";
  string quiet_str = "-q";

  if (argc < 3) { // print Usage information
    cout << "Usage: " << argv[0] << " [ -v | -q ] File1 File2\n";
    cout << "Option -v:\tverbose Messages\n";
    cout << "Option -q:\tquiet mode\n";
    cout << "File1:\t\tInitial Marking and P/N-Net in LoLA format\n";
    cout << "File2:\t\tFinal Marking(s) in LoLA format\n";
    std::exit(1);
  }

  if (argv[1] == verbose) { // check for options
    verb = true;
    file1 = argv[2];
    file2 = argv[3];
  } else {
    if (argv[1] == quiet_str) { // check for options
      quiet = true;
      file1 = argv[2];
      file2 = argv[3];
    } else { // no options
      file1 = argv[1];
      file2 = argv[2];
    }
  }
  
  InputCvs* in = new InputCvs;
  if (verb) { in->setVerbose(); }
  
  if (in->run(file1,file2)) {
    Cvs* cvs = new Cvs(in->getPredicateTransformerSet(),
		       in->getInitialConstraintSet(),
		       in->getFinalConstraintSet(),
		       in->getPlaceNames(),
		       in->getTransformerNames());
    if (verb) { cvs->setVerbose(); }
    if (quiet) { cvs->setQuiet(); }
    
    cvs->run();
    std::exit(0);
  } else { 
    cout << "Error happened while trying to read Petri-Net description "
	 << "and Test Marking(s) from "
	 << file1 << " and " << file2 << endl;
    std::exit(1);
  }
}
