#include "InputCG.h"
#include "CG.h"
#include "InputCvs.h"
#include "Cvs.h"

int main(int argc, char* argv[]) {
  bool verb = false; // verbose messages  
  bool quiet = false; // quiet mode
  char* file1;
  char* file2;
  string verbose = "-v";
  string quiet_str = "-q";

  bool cg_ok = false;
  bool cvs_ok = false;
  
  if (argc < 3) {
    cout << "Usage: " << argv[0] << " [ -v | -q ] File1 File2\n";
    cout << "Option -v:\tverbose Messages\n";
    cout << "Option -q:\tquiet mode\n";
    cout << "File1:\t\tInitial Marking and P/N-Net in LoLA format\n";
    cout << "File2:\t\tTest Marking(s) in LoLA format\n";
    std::exit(1);
  }
  
  if (argv[1] == verbose) {
    verb = true;
    file1 = argv[2];
    file2 = argv[3];
  } else {
    if (argv[1] == quiet_str) {
      quiet = true;
      file1 = argv[2];
      file2 = argv[3];
    } else {
      file1 = argv[1];
      file2 = argv[2];
    }
  }

  InputCG* incg = new InputCG;
  if (verb) { incg->setVerbose(); }
  
  CG* cg;
  
  if (incg->run(file1,file2)) {
    cg = new CG(incg->getTransitionSet(),incg->getMarking(),
		incg->getPlaceNames(),
		incg->getTransitionNames());
    
    if(verb) { cg->setVerbose(); }
    if (quiet) { cg->setQuiet(); }

    if (quiet) { cout << "Coverability graph:\t"; }
    
    if (cg->run_covers(incg->getTestMarkings())) { 
      cg_ok = true;
    }
  } else { 
    cout << "Coverability Graph:\n"
	 << "Error occured while trying to read Petri-Net description from "
	 << file1 << " and " << file2 << endl;
    std::exit(1);
  }
  
  delete incg;
  delete cg;
  
  InputCvs* in = new InputCvs;
  Cvs* cvs;
  
  if (in->run(file1,file2)) {
    cvs = new Cvs(in->getPredicateTransformerSet(),
		  in->getInitialConstraintSet(),
		  in->getFinalConstraintSet(),
		  in->getPlaceNames(),
		  in->getTransformerNames());

    if(verb) { cvs->setVerbose(); }
    if (quiet) { cvs->setQuiet(); }

    if (quiet) { cout << "Backward Reachability:\t"; }

    if (cvs->run()) {
      cvs_ok = true;
    }
    
  } else { 
    cout << "Backward Reachabilty Algorithm:\n"
	 << "Error occured while trying to read Petri-Net description from "
	 << file1 << " and " << file2 << endl;
    std::exit(1);
  }
  
  delete cvs;
  delete in;

  if (!quiet) { 
    cout << endl;
    if (cg_ok) {
      cout << "Coverability Graph: Final-Marking(s) are covered.\n";
    } else {
      cout << "Coverability Graph: Final-Marking(s) are not covered.\n";
    }
    
    if (cvs_ok) {
      cout << "Backward Reachability: Final Marking(s) are covered..\n";
    } else {
      cout << "Backward Reachability: Final Marking(s) are not covered.\n";
    }
  }
}
