#include "intSet.h"
#include "Node.h"
#include "NodeSet.h"
#include "Transition.h"
#include "TransitionSet.h"
#include "stringMap.h"


#ifndef CG_H
#define CG_H

/* 
   This class implements/builds the Coverability Graph (CG).
*/
class CG {
  /* internal variables */
  
  // this variables indicate how much messages to print
  bool verb;
  bool quiet;

  // store the time when process starts
  double timer;
  
  // the nodes building the CG
  NodeSet* nodes;
  
  // the transition set, used generate the CG
  TransitionSet* transitions;
  
  // string to integer mappings of place and transition names
  stringMap* placeNames;
  stringMap* transitionNames;
 
 // often needed/used iterators
  TransitionSet::iterator tsIT;
  NodeSet::iterator nodesIT;
  NodeSet::iterator testNodesIT;
  intSet::iterator idIT;
  
 public:
  CG(TransitionSet* T,Node* initialMarking,
     stringMap* inPlaceNames,stringMap* inTransNames); // constructor

  // set how much messages to print
  void setVerbose();
  void setQuiet();
  
  bool run(); // central method: run the CG-algorithm

  // check if this CG covers any of the input markings?
  bool covers(NodeSet* testNodes);
  
  // run CG-algorithm and check if CG covers any input-markings
  bool run_covers(NodeSet* testNodes);

  bool empty(); // is this CG empty, ie. does it have any nodes?

  string asString(); // return string-representation of this CG

  string getTime(); // return processed time as string

  // two places will be unioned to one,
  // marks will be saved in place1, place2 will be deleted
  void unionPlaces(string place1, string place2);
  void unionPlaces(int place1, int place2);

  // return names of known places
  stringMap* getPlaceNames();

  // return names of known transitions
  stringMap* getTransitionNames();
  
  // return reference to transition set
  TransitionSet* getTransitionSet();
};

#endif
