#include "Node.h"
#include "NodeSet.h"
#include "Transition.h"
#include "TransitionSet.h"
#include "stringMap.h"

#ifndef InputCG_H
#define InputCG_H

/* class InputCG: read Petri-Net description from files
   and provide final set of markings,
   initial set of markings and transition set.
*/
class InputCG {
 protected:
  bool verb; // indicates to print more information to console
  
  /* variables, to store parsed information:
     places, markings, transitions, place and transition names. */
  Node* marking;
  NodeSet* testMarkings;
  TransitionSet* tSet;
  stringMap* placeNames;
  stringMap* transitionNames;

  // helper methods  
  void error_exit(const string in, bool exit);
  string readFile(const char* file);
  
  // parse-method for place names
  void setPlaces(string in);
  
 public:
  InputCG(); // constructor

  /* provide access to parsed information*/
  TransitionSet* getTransitionSet();
  Node* getMarking();
  NodeSet* getTestMarkings();
  stringMap* getTransitionNames();
  stringMap* getPlaceNames();
  
  void setVerbose(); // print more messages to console
  
  // return string-represenation of parsed transitions and markings
  string asString(); 

  // methods, which actually do the parsing
  Node* parseMarking(string in,stringMap* places); 
  bool parseTransition(string in,TransitionSet* tSet,
		       stringMap* transitions,stringMap* places,bool exit);

  // run/initiate parsing of files
  bool run(const char* file,const char* file2);
};

#endif
