#include "Node.h"
#include <sstream>

// constructor, initialize internal variables
Node::Node() { 
  appliedTransitions = new intSet; 
  pre = new intSet; 
};

// is this node empty?
bool Node::isEmpty() { return marking.empty(); }

// return value at given place
int Node::get(int key) { return marking[key]; }

// delete value at given place
void Node::del(int key) { marking.erase(key); }

// add one more pair (predecessor node and transition ID)
void Node::setFather(int key,int value) { father[key] = value; }

// add given ID to the set of IDs of predecessor nodes
void Node::addPre(int in) { pre->add(in); }

// add ID of applied transition 
void Node::addAppliedTransitions(int in) { appliedTransitions->add(in); }

// has the Transition with given ID already been applied to this node?
bool Node::hasApplied(int in) { return appliedTransitions->has(in);}

// return reference to IDs of predecessor nodes
intSet* Node::getPredecessorsIds() { return pre; }

// set the value of given place/key to given value
void Node::set(int key,int value) { 
  if (value == 0) {
    this->del(key); // setting to 0 => deleting key
  } else {
    marking[key] = value;   
  }
}

// copy only Marking, not Predecessors, Fathers, applied Transitions...
Node* Node::copyMarking() {
  Node* newNode = new Node;
  for (i=marking.begin(); i!=marking.end(); ++i) {
    newNode->set(i->first,i->second);
  }
  return newNode;
}

// is this node smaller equal than given node?
bool Node::smallerEqualAs(Node* in) {
  int myVal = 0;
  int opVal = 0;
  for (i=marking.begin(); i!=marking.end(); ++i) {
    myVal = i->second;
    opVal = in->get(i->first);
    if ((myVal == INF) && (opVal != INF)) return false;
    if ((myVal != INF) && (opVal != INF) && (myVal > opVal)) return false; 
  }
  return true;
}

// is this node smaller equal than given node?
bool Node::operator<=(Node* in) {return this->smallerEqualAs(in); }

// is this node equal to given node?
bool Node::isEqual(Node* in){
  if (this->smallerEqualAs(in) && in->smallerEqualAs(this)) {
    return true;
  }
  return false;
}

// is this node equal to given node?
bool Node::operator==(Node* in) {return this->isEqual(in); }

// for any place, where the value of this Node is smaller than the
// value of the input Node, set the value of input node at that place to infinity
void Node::buildCoveringNode(Node* node) {
  int opVal = 0;
  int myKey = 0;
  int myVal = 0;

  // iterate through all places of this node
  // and compare to input-node
  for (i=marking.begin(); i!=marking.end(); ++i) {
    myKey = i->first;
    myVal = i->second;
    opVal = node->get(myKey);
    
    // if value of this node is smaller than input-node`s value
    // set input-node`s value to infinity
    if ((myVal != INF) && (opVal != INF) && (myVal < opVal)) {
      node->set(myKey,INF);
    }
  }
}

// return string-representation of this set
string Node::asString(stringMap* names) {
  ostringstream os;
  string name = "";
  for (i=marking.begin(); i!=marking.end(); ++i) {
    if (i->second != 0) { // skip 0-values
      name = names->getString(i->first);
      os << "(" << name << ",";
      
      if (i->second == Node::INF) {
	os << Node::INFc;
      } else {
	os << i->second;
      }
      os <<  ") ";
    }
  }
  os << endl;
  return os.str();
}

// add the marks of place2 to place1, and delete place2
void Node::unionPlaces(int place1, int place2) {
  int val1 = 0;
  int val2 = 0;
  val1 = this->get(place1);
  val2 = this->get(place2);
  int res = 0;

  // check if any value is Infinity
  if ((val1 == Node::INF) || (val2 == Node::INF)){
    // set value to Infinity
    res = Node::INF;
  } else {
    // add values of given places
    res = val1+val2;
  }
  
  // save marks in place1
  this->set(place1,res);

  // delete place2
  this->del(place2);
}
