#include <map>
#include <iostream>
#include "intSet.h"
#include "stringMap.h"

using namespace std;

#ifndef NODE_H
#define NODE_H

/* A Node is similar to a marking, 
   but it does a mapping from Places P to Integers or Omega(infinity),
   and to each node, the IDs of predecessor nodes, 
   and the IDs of applied transitions are saved.
*/
class Node {
  // internal container for mapping from places to values
  map<int,int> marking; 

  // save father.node.ID in key, and transition.ID in value
  map<int,int> father; 
  
  // store IDs of transitions already applied to this Node
  intSet* appliedTransitions; 
  
  // IDs of predecessor nodes
  intSet* pre; 
  
  map<int,int>::iterator i; // internal used iterator
 public:
  Node(); // constructor
  const static int INF = -1; // used as infinity
  const static char INFc = 'w'; // used as infinity in strings

  // set the value of given place/key to given value
  void set(int key,int value);

  bool isEmpty(); // is this node empty?

  // return value at given place
  int get(int key);

  // delete value at given place
  void del(int key);
  
  // add one more pair (predecessor node and transition ID)
  void setFather(int key,int value);
  
  // add given ID to the set of IDs of predecessor nodes
  void addPre(int in);
  
  // add ID of applied transition 
  void addAppliedTransitions(int in);
  
  // has the Transition with given ID already been applied to this node?
  bool hasApplied(int in);

  // return reference to IDs of predecessor nodes
  intSet* getPredecessorsIds();
  
  // return a new copy of the marking of this node,
  // not any predecessor information.
  Node* copyMarking();

  // is this node smaller equal than given node?
  bool smallerEqualAs(Node* in);
  bool operator<=(Node* in);
  
  // is this node equal to given node?
  bool isEqual(Node* in);
  bool operator==(Node* in);
  
  // for any place, where the value of this Node is smaller than the
  // value of the input Node, set the value of input node at that place to infinity
  void buildCoveringNode(Node* node);

  // return string-representation of this set
  string asString(stringMap* names); 

  // add the marks of place2 to place1, and delete place2
  void unionPlaces(int place1, int place2);
};
#endif
