#include "NodeSet.h"
#include <sstream>

NodeSet::NodeSet() { count = 0;} // constructor

// methods to point iterator to begin/end of this set
NodeSet::iterator NodeSet::begin() { return nSet.begin(); }
NodeSet::iterator NodeSet::end() { return nSet.end(); }

// methods to access the nodes via iterator
int NodeSet::getKey(NodeSet::iterator& iter) { return iter->first; }
Node* NodeSet::getNode(NodeSet::iterator& iter){ return iter->second; }

// access to nodes via their ID
Node* NodeSet::getNode(int id) { return nSet[id]; }

// is this set of nodes empty?
bool NodeSet::isEmpty() { return nSet.empty(); }

// internal add-method, which does not check,
// if there is a node equal to input-node
int NodeSet::add_intern(Node* in) {
  count++;
  nSet[count] = in;
  return count;
}

/* 
   Check if there is a node equal to input-node,
   if so return its ID, else add the node
   to this set and return its new ID
*/
int NodeSet::add(Node* in) {
  int id = 0;
  id = this->hasId(in);
  if (id == 0) {
    return this->add_intern(in);
  } else {
    // given node is already in NodeSet, just return its ID
    return id;
  }
}

// check if there is an node in this set which is equal to given input-node
bool NodeSet::has(Node* in) {
  for (i=nSet.begin(); i!=nSet.end(); ++i) { 
    if (*in == i->second) { 
      return true;
    }
  } 
  return false;
}

// return 0 if there is no node in this set, 
// which is equal to the given Node.
// else return the ID of the equal node
int NodeSet::hasId(Node* in) {
  for (i=nSet.begin(); i!=nSet.end(); ++i) { 
    if (*in == i->second) { 
      return i->first;
    }
  } 
  return 0;
}

// return IDs of all predecessor nodes of node with given ID,
// which are smaller than given node
intSet* NodeSet::getSmallerPredNodesIds(Node* node,int begin_id) {
  int id = 0;
  intSet* unvisited = new intSet; // IDs of unvisited nodes
  intSet* visited = new intSet;  // IDs of visited nodes

  // IDs of predecessor nodes, which are smaller than given node
  intSet* res = new intSet; 
  
  unvisited->add(begin_id); // start with given ID
  
  // visit all predecessor nodes until the root node
  while(!unvisited->isEmpty()) {
    id = unvisited->pop();

    // check if this node is not already visited
    if (!(visited->has(id) || (res->has(id)))) {

      // check if unvisted node is smaller than given input-node
      if ((this->getNode(id))->smallerEqualAs(node)) {
	res->add(id); // add visited node to result set
      } else {
	visited->add(id); // visited node is not smaller
      }
      // add the IDs of the predecessor nodes of visited node to the unvisited nodes
      unvisited->add((this->getNode(id))->getPredecessorsIds());
    }
  }
  delete unvisited;
  delete visited;
  return res;
}

// return string-representation of this set
string NodeSet::asString(stringMap* names) {
  ostringstream os;
  for (i=nSet.begin(); i!=nSet.end(); ++i) { 
    os << "Node with ID " << i->first << " : ";
    os << (i->second)->asString(names); 
  } 
  return os.str();
}

// in all nodes given places will be united to the first one
void NodeSet::unionPlaces(int place1, int place2) {
  for (i=nSet.begin(); i!=nSet.end(); ++i) { 
    i->second->unionPlaces(place1,place2);
  }
}
