#include <map>
#include <iostream>
#include "intSet.h"
#include "Node.h"
#include "stringMap.h"
using namespace std;

#ifndef NodeSet_H
#define NodeSet_H

/* this class provides a container for the nodes
   of the coverability graph
   and iterator-methods for access to nodes
   within the set of nodes.
*/
class NodeSet {
  map<int,Node*> nSet; // internal container for nodes
  map<int,Node*>::iterator i; // internal used iterator
  int count; // used to assign a unique ID to each node
 protected:
  int add_intern(Node* in); // internal add-method
 public:
  NodeSet(); // constructor
  
  // iterator: type and methods  
  typedef map<int,Node*>::iterator iterator;
  iterator begin();
  iterator end();

  // methods to access the nodes via iterator
  int getKey(NodeSet::iterator& iter);
  Node* getNode(NodeSet::iterator& iter);

  // access to nodes via their ID
  Node* getNode(int id);

  bool isEmpty(); // is this set of nodes empty?

  // add a node to this set, and assign a unique ID to it
  int add(Node* in); 
  
  // check if there is an node in this set which is equal to given input-node
  bool has(Node* in);

  // check if there is an node in this set which is equal to given input-node
  // if so return its ID, else 0
  int hasId(Node* in);
  
  // check for all ids (Nodes with these ids), if they are smaller
  // equal "node", when so add them to return intSet.

  // return IDs of all predecessor nodes of node with given ID,
  // which are smaller than given node
  intSet* getSmallerPredNodesIds(Node* node,int begin_id);
  
  // return string-representation of this set
  string asString(stringMap* names); 

  // in all nodes given places will be united to the first one
  void unionPlaces(int place1, int place2);
};
#endif
