#include "Transition.h"
#include <sstream>

Transition::Transition() {} // constructor

// set place-value-pair for in-function
void Transition::setIn(int key,int value) { in[key] = value; }

// set place-value-pair for out-function
void Transition::setOut(int key,int value) { out[key] = value; }

// get value at given place for in-function
int Transition::getIn(int key) { return in[key]; }

// get value at given place for out-function
int Transition::getOut(int key) { return out[key]; }

// delete value at given place for in-function
void Transition::delIn(int key) { in.erase(key); }

// delete value at given place for out-function
void Transition::delOut(int key) { out.erase(key); }

// is this transition empty, i.e. in- and out-function empty?
bool Transition::empty() { return (in.empty() && out.empty()); }

/* can this transition be fired on given node,
   i.e. for each value at given place for the in-function,
   the value for this place in the node is greater/equal
*/
bool Transition::applicable(Node* input){
  int myVal = 0;
  int opVal = 0;
  
  // check for each place-value-pair of the in-function...
  for (i=in.begin(); i!=in.end(); ++i) {
    myVal = i->second;
    opVal = input->get(i->first);
    
    // ... if the value at given place of node is greater/equal
    if ((opVal != Node::INF) && (myVal > opVal)) {
      
      // in this case, transition can not be fired on input-node
      return false;
    }
  }
  return true;
}

// return new resulting Node, after Transition is applied to input Node
Node* Transition::apply(Node* input){
  int temp = 0;
  int opVal = 0;

  Node* newNode = input->copyMarking(); // original node is copied

  // first add to all places, the values of the out-function
  for (i=out.begin(); i!=out.end(); ++i) {
    opVal = newNode->get(i->first);
    if (opVal != Node::INF) { 
      newNode->set(i->first,opVal+(i->second)); // add the values
    }
  }
  
  // then subtract for all places, the values of the in-function
  for (i=in.begin(); i!=in.end(); ++i) {
    opVal = newNode->get(i->first); // get the value of Node
    if (opVal != Node::INF) {// subtract only, when value is not INFINITY
      temp = opVal-(i->second);
      if (temp <= 0) { // transition subtracts more marks than available
	newNode->del(i->first);
      } else {
	newNode->set(i->first,temp); // update place-value
      }
    }
  }
  return newNode; // return resulting node/marking
}

// return string-representation of this transtion
string Transition::asString(stringMap* placeNames) {
  stringstream os;
  os << "\tIn Function: ";
  for (i=in.begin(); i!=in.end(); ++i) {
    os << '(' << placeNames->getString(i->first) << ',' << i->second << ") ";
  }
  //os << endl;
  os << "\tOut Function: ";
  for (i=out.begin(); i!=out.end(); ++i) {
    os << '(' << placeNames->getString(i->first) << ',' << i->second << ") ";
  }
  os << endl;
  return os.str();
}

// two given places will be unioned to the first given place (marks are added)
void Transition::unionPlaces(int place1, int place2) {
  this->setIn(place1,
	      this->getIn(place1)+this->getIn(place2));
  this->delIn(place2);
  this->setOut(place1,
	       this->getOut(place1)+this->getOut(place2));
  this->delOut(place2);
}
