#include <map>
#include <iostream>
#include "Node.h"
#include "stringMap.h"

using namespace std;

#ifndef TRANSITION_H
#define TRANSITION_H

/* Central point for Transitions is an OUT- and an IN-function,
   stating how many tokens are added/removed at each place,
   when firing a transition on a given marking.
*/
class Transition {
 protected:
  map<int,int> in; // interal container for in-Function
  map<int,int> out;// interal container for out-Function
  map<int,int>::iterator i; // internal used iterator
 public:
  Transition(); // constructor

  void setIn(int key,int value); // set place-value-pair for in-function
  void setOut(int key,int value);// set place-value-pair for out-function
  int getIn(int key); // get value at given place for in-function
  int getOut(int key);// get value at given place for out-function
  void delIn(int key);// delete value at given place for in-function
  void delOut(int key);// delete value at given place for out-function
  
  bool empty(); // is this transition empty, i.e. in- and out-function empty?
 
  // can this transition be fired on given node,
  // i.e. for each value at given place for the in-function,
  // the value for this place in the node is greater/equal
  bool applicable(Node* input); 

  // this transition is fired on given input-node,
  // and resulting (new) node/marking is returned.
  Node* apply(Node* input);

  // return string-representation of this transtion
  string asString(stringMap* placeNames);

  // two given places will be unioned to the first given place (marks are added)
  void unionPlaces(int place1, int place2);
};
#endif
