#include <map>
#include <iostream>
#include "Node.h"
#include "Transition.h"
#include "stringMap.h"

using namespace std;

#ifndef TRANSITIONSET_H
#define TRANSITIONSET_H

/* This class provides a containter for transitions.
   The transitions can be accessed via their ID
   or via iterators.
 */
class TransitionSet {
 protected:
  map<int,Transition*> transitionSet; // internal container
  map<int,Transition*>::iterator i; // internal used iterator
  int count; // used to assign unique IDs to the transitions
 public:
  TransitionSet(); // constructor

  /* iterator: type and methods,
     for easily iterating over this set of transitions*/
  typedef map<int,Transition*>::iterator iterator;
  iterator begin(); // point the iterator to the first element
  iterator end(); // point iterator to end of this set
  
  bool isEmpty(); // is this set empty?
  
  // provide access to a transition via its ID
  Transition* getTransition(int id);

  // provide access to transitions via iterator
  Transition* getTransition(TransitionSet::iterator iter);
  int getID(TransitionSet::iterator iter);

  // add a transition to this set
  int add(Transition* in);
  
  // return string-representation of this set
  string asString(stringMap* placeNames,stringMap* transNames);
  
  // in all transitions given places will be united to the first one
  void unionPlaces(int place1, int place2);
};

#endif
