/* Main program (non-interactive/batch) for the coverability graph (CG) algorithm.
   2 Files in LoLA-Format are needed, 
   the first having the transition-net and initial marking,
   the second file having the final- or test-marking.
*/

#include "InputCG.h"
#include "CG.h"

/* get input-files from command-line,
   and apply coverability-graph-algorithm
*/
int main(int argc, char* argv[]) {

  // store command-line input
  bool verb = false; // verbose messages  
  bool quiet = false; // quiet mode
  
  char* file1; // store 1. file name
  char* file2; // store 2. file name
  
  // used for comparison of commandline-input
  string verbose = "-v";
  string quiet_str = "-q";
  
  // print usage information
  if (argc < 3) {
    cout << "Usage: " << argv[0] << " [ -v | -q ] File1 File2\n";
    cout << "Options:\n" << "\t -v: verbose\n";
    cout << "\t -q: quiet mode\n";
    cout << "Files:\n";
    cout << "\t File1: Initial Marking and P/N-Net in LoLA format\n";
    cout << "\t File2: Test Marking(s) in LoLA format\n";
    std::exit(1);
  }

  // check commandline-options
  if (argv[1] == verbose) {
    verb = true;
    file1 = argv[2];
    file2 = argv[3];
  } else {
    if (argv[1] == quiet_str) {
      quiet = true;
      file1 = argv[2];
      file2 = argv[3];
    } else {
      file1 = argv[1];
      file2 = argv[2];
    }
  }
  
  InputCG* in = new InputCG;
  if (verb) { in->setVerbose(); }
  
  // running parsing on files
  if (in->run(file1,file2)) {

    // initalize graph
    CG* cg = new CG(in->getTransitionSet(),
		    in->getMarking(),
		    in->getPlaceNames(),
		    in->getTransitionNames());
    
    // apply options
    if(verb) { cg->setVerbose(); }
    if (quiet) { cg->setQuiet(); }
    
    // run CG-algorithm
    cg->run_covers(in->getTestMarkings());
    
  } else { 
    cout << "Error: Could not read Petri-Net description and Test Markings from ";
    cout << file1 << " and " << file2 << endl;
    std::exit(1);
  }
}
