

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Globals.h"
#include "PTNet.h"
#include "GraphGrammar.h"
#include "GTXLReader.h"
#include "GXLWriter.h"
#include "Petrigraph.h"
#include "aunfold.h"

#include <iostream>
#include <cstdlib>

#include <time.h>

using namespace aunfold;
using namespace std;

// Added new option -nc (Barbara, 28-05-03)

int main(int argc, char**argv)
{

const char* usage = "Usage: aunfold [-q] [-d] [-t] [-nc] [-nr] \
[-depth=k | -nfold=k] [-step=n] [-nmapping] infile outfile\n\
\nOptions:\n\n\
-q quiet      quiet mode, disable verbose output\n\
-d debug      debug mode, show many debugging information\n\
-t trace      include each step of calculation into output file\n\
-nc           do not check covering information\n\
-nr           without irredundancy condition \n\
-depth=k      build over-approximation C(k) \n\
-step=n       make only n steps of the algorithm\n\
-nfold=k      no folding steps with maximal depth of unfolding \n\
-nmapping     without mapping functions\n\
-timeout=n    timeout atfer n sec\n\n";

  // parse args:
  int i;

  for(i=1; i<argc; i++)
    {
      if(!strcmp(argv[i],"-q")) // quiet mode
	quiet = true;
      else if(!strcmp(argv[i],"-d")) // debug output
	debug = true;
      else if(!strcmp(argv[i],"-t")) // trace
	trace = true;
      else if(!strcmp(argv[i],"-nc")) // do not check covering
	ncov = true;
			else if(!strcmp(argv[i],"-nr")) // do not check irredundancy condition
	red = false;
			else if(!strcmp(argv[i],"-nmapping")) // without mapping functions
	mapping = false;
			else if(strstr(argv[i],"-depth=")==argv[i])
			{
        char buffer[10];
				strcpy(buffer, argv[i]+7);
				depth = atoi(buffer);

				if(depth<0)
				{
					cout << "depth must be some positive number or 0" << "\n";
					return 1;
				}
			}
			else if(strstr(argv[i], "-nfold=")==argv[i]) // no folding steps
			{
				if(depth>0)
				{
					cout << "-depth and -nfold cannot be defined together";
					return 1;
				}
				
				nfold = true;
				
        char buffer[10];
				strcpy(buffer, argv[i]+7);
				unfdepth = atoi(buffer);

				if(unfdepth<=0)
				{
					cout << "unfolding depth must be some positive number" << "\n";
					return 1;
				}
					
			}
			else if(strstr(argv[i],"-step=")==argv[i])
			{
        char buffer[10];
				strcpy(buffer, argv[i]+6);
				maxstep = atoi(buffer);

				if(maxstep<=0)
				{
					cout << "step must be some positive number" << "\n";
					return 1;
				}
			}
                        else if(strstr(argv[i],"-timeout=")==argv[i])
			{
        char buffer[10];
				strcpy(buffer, argv[i]+9);
				timeout = atoi(buffer);

				if(timeout<=0)
				{
					cout << "timeout must be some positive number" << "\n";
					return 1;
				}
			}

      else if(*argv[i] == '-')
	{
	  cout << "Unknown option: " << argv[i] << "\n" << usage;
	  return 1;
	}
      else
	break;
    }
  if(i > argc-2)
    {
      cout << usage;
      return 1;
    }

  char* infile = argv[i];
  char* outfile = argv[i+1];

  GTXLReader reader(infile);
  GraphGrammar g = reader.read();

  if(debug)
    {
      cout << g << "\n";
    }

  GXLWriter writer(outfile);
  // try if we can really write _before_ starting the calculation:
  try
    {
      writer.write();
    }
  catch(exception e)
    {
      cerr << "Error while writing output file\n";
    }
      
  try
  {
    double timer = double(clock()) / CLOCKS_PER_SEC;
    Petrigraph pg (approximatedUnfolding(g,writer,timer));
    writer.addGraph(pg,"result");
    double timeused = double(clock()) / CLOCKS_PER_SEC - timer;
    if(timeused<0)
	timeused=0.0;
    cout<< "time used: " << timeused << " sek \n";

    // petrigraph size
    cout << "edges: " << pg.getHypergraph().getEdges().size();
    cout << ", vertices: " << pg.getHypergraph().getVertices().size();
    cout << ", transitions: " << pg.getPTNet().getTransitions().size() << "\n";
  }
  
  catch (exception& e)
  {
    cerr << e.what() << "\n";
  }

  try
    {
      writer.write();
    }
  catch(exception e)
    {
      cerr << "Error while writing output file\n";
    }


  return EXIT_SUCCESS;

}

 

