
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <stdlib.h>

#include <sys/types.h>
#include <unistd.h>
#include <dirent.h>

#include "GraphGrammar.h"
#include "GTXLReader.h"
#include "ConverterToNeato.h"

using namespace aunfold;
using namespace std;

int spawn (char* program, char** arg_list)
  {
		pid_t child_pid;

		/* Duplicate this process.  */
		child_pid = fork ();
		if (child_pid != 0)
			/* This is the parent process.  */
			return child_pid;
		else
	  {
			/* Now execute PROGRAM, searching for it in the path.  */
			execvp (program, arg_list);
			/* The execvp function returns only if an error occurs.  */
			fprintf (stderr, "an error occurred in execvp\n");
			abort ();
		}
  }

  // apply neato to "files" directory
  void callNeato(string output, int count)
  { 
    // initial graph
    string inis = output + (string)"/ini.dot";
    char* arg_list[] = {
    "neato",
    "-Tps",
    "-Goverlap=false",
    "-Gsplines=true",
    (char*)inis.c_str(),
    NULL
    };
    
		string inips = output + (string)"/ini.ps";
		freopen (inips.c_str(),"w",stdout);

		spawn ("neato", arg_list);

		// rules

	  string sel = "l.dot";
	  string ser = "r.dot";
	  string sfl = output + "/";
	  string sln = "l.ps";
	  string srn = "r.ps";

    for(int counter = 1; counter <=count; counter++)
    {
			char buf[16];
			sprintf(buf, "%d", counter);
			string sb(buf);

			string sl = sfl + sb + sel;
			string sr = sfl + sb + ser;

			string sl1 = sfl + sb + sln;
			string sr1 = sfl + sb + srn;

			// left
			char* arg_list_left[] = {
					"neato",
					"-Tps",
					"-Goverlap=false",
					"-Gsplines=true",
					(char *)sl.c_str(),
					NULL
			};

			freopen (sl1.c_str(),"w",stdout);

			spawn ("neato", arg_list_left);

			// right
			char * arg_list_right[] = {
					"neato",
					"-Tps",
					"-Goverlap=false",
					"-Gsplines=true",
					(char *)sr.c_str(),
					NULL
			};

			freopen (sr1.c_str(),"w",stdout);

			spawn ("neato", arg_list_right);
    }

  }

  void makeLaTex(char * outputfile, int count, GraphGrammar & ggr, float size)
  {
		 string outputtex = (string)outputfile+".tex";
		 ofstream of(outputtex.c_str());

                 string output = (string)outputfile;

                 char * out = new char[output.size() + 1];
                 strcpy(out, output.c_str());
  
                 char * pch;
                 pch = strtok(out,"/");
                 while(pch!=NULL)
                 {
                    output = (string)pch;
                    pch = strtok(NULL, "/");
                 }

		 // write head
		 of << "\\documentclass{article}\n \\newcommand{\\epsff}[1]{\\scalebox{";
		 of << size;
		 of << "}{\\epsffile{#1}}} \n \\usepackage{epsf, graphicx} \n \\begin{document} \n ";

		 // initial graph
                 of << "Initial graph:\\qquad\n";
		 of << "\\begin{tabular}[c]{r} \n \\epsff{" << output << "_files/ini.ps} \n \\end{tabular} \\\\ \n";
		 
		 string sfl = (string)output + "_files/";
		 string sln = "l.ps";
		 string srn = "r.ps";

		 // rules
		 map<IDTYPE,TransformationRule> rules = ggr.getRules();

		 map<IDTYPE,TransformationRule>::iterator it = rules.begin();

		 // rules
		 for(int i=1; i<=count; i++)
		 {
			 char buf[16];
			 sprintf(buf, "%d", i);
			 string sb(buf);

			 string sl = sfl + sb + sln;
			 string sr = sfl + sb + srn;

			 // name of rule
			 string s = it->second.getName();
			 it++;

			 of << "\\bigskip\\hrule\\bigskip\n";

                         of << s.c_str() << ": \\\\\n";

			 // left
			 of << "\\begin{tabular}{rcl} \n \\begin{tabular}[c]{r} \n \\epsff{" << sl.c_str() << "} \n \\end{tabular} & \n";

			 // arrow
			 of << "$\\Longrightarrow$ & \n";

			 // right
			 of << "\\begin{tabular}[c]{l} \n \\epsff{" << sr.c_str() << "} \n \\end{tabular} \n \\end{tabular} \\\\ \n";

		 }

		 // write bottom
		 of << "\\end{document} \n";
  }


int main(int argc, char *argv[])
{
  const char * usage = "Usage: rules2LaTex [-size=n] inputfile outputfile";

  if(argc<3)
  {
	cout << usage << endl;
	return 1;
  }

  int i;

  float size = 0.4;

  if(argv[1][0]=='-')
  {
    i=2;
    if(strlen(argv[1])>=7)
      size = atof(&argv[1][6]);
  }
  else
    i=1;

  char * outputfile = argv[i+1];

  GTXLReader reader(argv[i]);

  GraphGrammar ggr = reader.read();

  string output;
  output = (string)strtok(outputfile, ".")+"_files";        
   
  string makedir = "mkdir " + output;
  
  if(opendir(output.c_str())==NULL)
    system(makedir.c_str());

  ConverterToNeato ctn(ggr, output);

  ctn.convert();

  int count = ggr.getRules().size();
  
  callNeato(output, count);

  makeLaTex(outputfile, count, ggr, size);

  return EXIT_SUCCESS;
}
